#!/usr/bin/env python
import os
import json
import argparse
from tripal import TripalAuth, TripalAnalysis, TripalInstance

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Creates an analysis into Tripal')
    TripalAuth(parser)
    TripalAnalysis(parser)

    args = parser.parse_args()

    ti = TripalInstance(args.tripal, args.username, args.password)

    params = ti.analysis.getBasePayload(args)

    params.update({
        'type': 'chado_analysis',
    })

    res = ti.analysis.addAnalysis(params)

    print "New analysis created with Node ID: %s" % res['nid']



#!/usr/bin/env python
import json
import argparse
from tripal import TripalAuth, TripalInstance

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Get details about a specific Tripal job')
    TripalAuth(parser)
    parser.add_argument('job_id', type=int, help='ID of the job to retrieve')
    args = parser.parse_args()

    ti = TripalInstance(args.tripal, args.username, args.password)

    print json.dumps(ti.jobs.getJob(args.job_id), indent=2)



#!/usr/bin/env python
import os
import json
import argparse
from chado import ChadoAuth, ChadoInstance, Organism, Analysis
from tripal import TripalAuth, TripalInstance

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Loads a GFF3 file into Tripal')
    TripalAuth(parser)
    parser.add_argument('gff', help='Path to the GFF3 file to load')
    parser.add_argument('--job-name', help='Name of the job (default=\'Import GFF3 file: <gff3_file_name>\')')
    parser.add_argument('--organism', required=True, help='Organism common name')
    parser.add_argument('--analysis', required=True, help='Analysis name')
    parser.add_argument('--import-mode', choices=['add_only', 'update', 'refresh', 'remove'], default='update', help='Import mode, default=update (add_only=existing features won\'t be touched, update=existing features will be updated and obsolete attributes kept, refresh=existing features will be updated and obsolete attributes removed, remove=features present in the db and in the GFF3 file will be reomved)')
    parser.add_argument('--target-organism-id', type=int, help='In case of Target attribute in the GFF3, choose the organism to which target sequences belong. Select this only if target sequences belong to a different organism than the one specified with --organism-id. And only choose an organism here if all of the target sequences belong to the same species. If the targets in the GFF file belong to multiple different species then the organism must be specified using the \'target_organism=genus:species\' attribute in the GFF file.')
    parser.add_argument('--target-type', help='In case of Target attribute in the GFF3, if the unique name for a target sequence is not unique (e.g. a protein and an mRNA have the same name) then you must specify the type for all targets in the GFF file. If the targets are of different types then the type must be specified using the \'target_type=type\' attribute in the GFF file. This must be a valid Sequence Ontology (SO) term.')
    parser.add_argument('--target-create', action='store_true', help='In case of Target attribute in the GFF3, if the target feature cannot be found, create one using the organism and type specified above, or using the \'target_organism\' and \'target_type\' fields specified in the GFF file. Values specified in the GFF file take precedence over those specified above.')
    parser.add_argument('--start-line', type=int, help='The line in the GFF file where importing should start')
    parser.add_argument('--landmark-type', help='A Sequence Ontology type for the landmark sequences in the GFF fie (e.g. \'chromosome\'). If the GFF file contains a \'##sequence-region\' line that describes the landmark sequences to which all others are aligned and a type is provided here then the features will be created if they do not already exist. If they do exist then this field is not used')
    parser.add_argument('--alt-id-attr', help='Sometimes lines in the GFF file are missing the required ID attribute that specifies the unique name of the feature, but there may be another attribute that can uniquely identify the feature. If so, you may specify the name of the attribute to use for the name.')
    parser.add_argument('--create-organism', action='store_true', help='The Tripal GFF loader supports the "organism" attribute. This allows features of a different organism to be aligned to the landmark sequence of another species. The format of the attribute is "organism=[genus]:[species]", where [genus] is the organism\'s genus and [species] is the species name. Check this box to automatically add the organism to the database if it does not already exists. Otherwise lines with an organism attribute where the organism is not present in the database will be skipped.')
    parser.add_argument('--re-mrna', help='Regular expression for the mRNA name')
    parser.add_argument('--re-protein', help='Replacement string for the protein name')

    ChadoAuth(parser)
    args = parser.parse_args()

    ti = TripalInstance(args.tripal, args.username, args.password)

    ci = ChadoInstance(args.dbhost, args.dbname, args.dbuser, args.dbpass, args.dbschema, args.debug)
    ci.connect()

    # check if the organism exists and get its id
    resdb = ci.session.query(Organism).filter_by(common_name = args.organism)
    if not resdb.count():
        raise Exception("Could not find the organism %s in the database %s" % (args.organism, ci._engine.url))
    organism_id = resdb.one().organism_id

    # check if the analysis exists and get its id
    resdb = ci.session.query(Analysis).filter_by(name = args.analysis)
    if not resdb.count():
        raise Exception("Could not find the analysis %s in the database %s" % (args.analysis, ci._engine.url))
    analysis_id = resdb.one().analysis_id

    job_name = args.job_name
    if not job_name:
        job_name = 'Import GFF3 file: %s' % os.path.basename(args.gff)

    transaction = 1 # use transaction or not, no reason to disable this

    job_args = [args.gff, organism_id, analysis_id, int(args.import_mode == 'add_only'),
                int(args.import_mode == 'update'), int(args.import_mode == 'refresh'), int(args.import_mode == 'remove'),
                transaction, args.target_organism_id, args.target_type, int(args.target_create), args.start_line,
                args.landmark_type, args.alt_id_attr, int(args.create_organism), args.re_mrna, args.re_protein]

    r = ti.jobs.addJob(job_name, 'tripal_feature', 'tripal_feature_load_gff3', job_args)
    print 'Load GFF3 job scheduled with id %s' % r['job_id']



#!/usr/bin/env python
import os
import json
import argparse
from chado import ChadoAuth, ChadoInstance, Organism, Analysis
from tripal import TripalAuth, TripalInstance

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Loads a fasta file into Tripal')
    TripalAuth(parser)
    parser.add_argument('fasta', help='Path to the fasta file to load')
    parser.add_argument('--job-name', help='Name of the job (default=\'Import FASTA file: <fasta_file_name>\')')
    parser.add_argument('--organism', required=True, help='Organism common name')
    parser.add_argument('--analysis', required=True, help='Analysis name')
    parser.add_argument('--sequence-type', help='Sequence type (default: contig)', default='contig')
    parser.add_argument('--re-name', help='Regular expression for the name', default='')
    parser.add_argument('--re-uniquename', help='Regular expression for the unique name', default='')
    parser.add_argument('--db-ext-id', help='External DB ID', default='')
    parser.add_argument('--re-accession', help='Regular expression for the accession from external DB', default='')
    parser.add_argument('--rel-type', help='Relation type', choices=['part_of', 'derives_from'], default='')
    parser.add_argument('--rel-subject-re', help='Relation subject regular expression (used to extract id of related entity)', default='')
    parser.add_argument('--rel-subject-type', help='Relation subject type (must match already loaded data, e.g. mRNA)', default='')
    parser.add_argument('--method', help='Insertion method', choices=['Insert only', 'Update only', 'Insert and update'], default='Insert and update')
    parser.add_argument('--match-type', help='Match type for already loaded features (used for "Update only" or "Insert and update" methods)', choices=['Name', 'Unique name'], default='Unique name')

    ChadoAuth(parser)
    args = parser.parse_args()

    ti = TripalInstance(args.tripal, args.username, args.password)

    ci = ChadoInstance(args.dbhost, args.dbname, args.dbuser, args.dbpass, args.dbschema, args.debug)
    ci.connect()

    # check if the organism exists and get its id
    resdb = ci.session.query(Organism).filter_by(common_name = args.organism)
    if not resdb.count():
        raise Exception("Could not find the organism %s in the database %s" % (args.organism, ci._engine.url))
    organism_id = resdb.one().organism_id

    # check if the analysis exists and get its id
    resdb = ci.session.query(Analysis).filter_by(name = args.analysis)
    if not resdb.count():
        raise Exception("Could not find the analysis %s in the database %s" % (args.analysis, ci._engine.url))
    analysis_id = resdb.one().analysis_id

    job_name = args.job_name
    if not job_name:
        job_name = 'Import FASTA file: %s' % os.path.basename(args.fasta)

    uid = 1 # user id is not really used by the loader, 1 is admin user

    job_args = [args.fasta, organism_id, args.sequence_type, args.re_name, args.re_uniquename, args.re_accession,
                args.db_ext_id, args.rel_type, args.rel_subject_re, args.rel_subject_type,
                args.method, uid, analysis_id, args.match_type]

    r = ti.jobs.addJob(job_name, 'tripal_feature', 'tripal_feature_load_fasta', job_args)
    print 'Load fasta job scheduled with id %s' % r['job_id']



#!/usr/bin/env python
import os
import json
import argparse
from chado import ChadoAuth, ChadoInstance
from tripal import TripalAuth, TripalAnalysis, TripalInstance

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Loads Blast2GO results into Tripal (requires tripal_analysis_go module)')
    TripalAuth(parser)
    TripalAnalysis(parser)
    parser.add_argument('gaf', help='Path to the GAF file to load (single XML file, or directory containing multiple XML files)')
    parser.add_argument('--gaf-ext', help='If looking for files in a directory, extension of the GAF files')
    parser.add_argument('--query-type', help='The feature type (e.g. \'gene\', \'mRNA\', \'contig\') of the query. It must be a valid Sequence Ontology term.')
    parser.add_argument('--query-uniquename', action='store_true', help='Use this if the --query-re regular expression matches unique names instead of names in the database.')
    parser.add_argument('--method', choices=['add', 'remove'], default='add', help='Import method')
    parser.add_argument('--re-name', help='Regular expression to extract the feature name from GAF file.')

    args = parser.parse_args()

    ti = TripalInstance(args.tripal, args.username, args.password)

    params = ti.analysis.getBasePayload(args)

    params.update({
        'type': 'chado_analysis_go',
        'gaf_file': args.gaf,
        'gaf_file_ext': args.gaf_ext,
        'seq_type': args.query_type,
        'query_uniquename': args.query_uniquename,
        'method': args.method,
        're_name': args.re_name,
        'gojob': 1,
    })

    res = ti.analysis.addAnalysis(params)

    print "New GO analysis created with ID: %s" % res['nid']



#!/usr/bin/env python
import os
import json
import argparse
from chado import ChadoAuth, ChadoInstance, Db
from tripal import TripalAuth, TripalAnalysis, TripalInstance

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Loads Blast results into Tripal (requires tripal_analysis_blast module)')
    TripalAuth(parser)
    TripalAnalysis(parser)
    parser.add_argument('blast', help='Path to the Blast file to load (single XML file, or directory containing multiple XML files)')
    parser.add_argument('--blast-ext', help='If looking for files in a directory, extension of the blast result files')
    parser.add_argument('--blastdb', required=True, help='Name of the database blasted against (must be in the Chado db table)')
    parser.add_argument('--blast-parameters', help='Blast parameters used to produce these results')
    parser.add_argument('--max-hits', type=int, default=25, help='Maximum number of hits kept (default=25)')
    parser.add_argument('--query-re', help='The regular expression that can uniquely identify the query name. This parameters is required if the feature name is not the first word in the blast query name.')
    parser.add_argument('--query-type', help='The feature type (e.g. \'gene\', \'mRNA\', \'contig\') of the query. It must be a valid Sequence Ontology term.')
    parser.add_argument('--query-uniquename', action='store_true', help='Use this if the --query-re regular expression matches unique names instead of names in the database.')
    parser.add_argument('--is-concat', action='store_true', help='If the blast result file is simply a list of concatenated blast results.')
    parser.add_argument('--search-keywords', action='store_true', help='Extract keywords for Tripal search')

    ChadoAuth(parser)
    args = parser.parse_args()

    ci = ChadoInstance(args.dbhost, args.dbname, args.dbuser, args.dbpass, args.dbschema, args.debug)

    ci.connect()

    ti = TripalInstance(args.tripal, args.username, args.password)

    # check if the blastdb exists and get its id
    resdb = ci.session.query(Db).filter_by(name = args.blastdb)
    if not resdb.count():
        raise Exception("Could not find the blastdb %s in the database %s" % (args.blastdb, ci._engine.url))
    blastdb_id = resdb.one().db_id

    params = ti.analysis.getBasePayload(args)

    params.update({
        'type': 'chado_analysis_blast',
        'blastdb': blastdb_id,
        'blastfile': args.blast,
        'blastfile_ext': args.blast_ext,
        'blastjob': 1, # no reason to not launch a job
        'blastparameters': args.blast_parameters,
        'query_re': args.query_re,
        'query_type': args.query_type,
        'query_uniquename': args.query_uniquename,
        'is_concat': int(args.is_concat),
        'search_keywords': int(args.search_keywords),
    })

    res = ti.analysis.addAnalysis(params)

    print "New Blast analysis created with ID: %s" % res['nid']



#!/usr/bin/env python
import json
import argparse
from collections import OrderedDict
from chado import ChadoAuth, ChadoInstance, Analysis
from tripal import TripalAuth, TripalInstance

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Synchronize analysis')
    TripalAuth(parser)
    parser.add_argument('--job-name', help='Name of the job (default=\'Sync Analysis\')')
    parser.add_argument('--analysis', required=True, help='Analysis name')

    ChadoAuth(parser)
    args = parser.parse_args()

    ti = TripalInstance(args.tripal, args.username, args.password)

    ci = ChadoInstance(args.dbhost, args.dbname, args.dbuser, args.dbpass, args.dbschema, args.debug)
    ci.connect()

    # check if the analysis exists and get its id
    resdb = ci.session.query(Analysis).filter_by(name = args.analysis)
    if not resdb.count():
        raise Exception("Could not find the analysis %s in the database %s" % (args.analysis, ci._engine.url))
    analysis_id = resdb.one().analysis_id

    job_name = args.job_name
    if not job_name:
        job_name = 'Sync Analysis'

    transaction = 1 # use transaction or not, no reason to disable this

    job_args = OrderedDict()
    job_args['base_table'] = 'analysis'
    job_args['max_sync'] = ''
    job_args['organism_id'] = ''
    job_args['types'] = []
    job_args['ids'] = [analysis_id]
    job_args['linking_table'] = 'chado_analysis'
    job_args['node_type'] = 'chado_analysis'

    r = ti.jobs.addJob(job_name, 'chado_feature', 'chado_node_sync_records', job_args)
    print 'Sync analysis job scheduled with id %s' % r['job_id']



#!/usr/bin/env python
import os
import json
import argparse
from chado import ChadoAuth, ChadoInstance
from tripal import TripalAuth, TripalAnalysis, TripalInstance

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Loads InterPro results into Tripal (requires tripal_analysis_interpro module)')
    TripalAuth(parser)
    TripalAnalysis(parser)
    parser.add_argument('interpro', help='Path to the InterProScan file to load (single XML file, or directory containing multiple XML files)')
    parser.add_argument('--interpro-parameters', help='InterProScan parameters used to produce these results')
    parser.add_argument('--parse-go', action='store_true', help='Load GO terms to the database')
    parser.add_argument('--query-re', help='The regular expression that can uniquely identify the query name. This parameters is required if the feature name is not the first word in the blast query name.')
    parser.add_argument('--query-type', help='The feature type (e.g. \'gene\', \'mRNA\', \'contig\') of the query. It must be a valid Sequence Ontology term.')
    parser.add_argument('--query-uniquename', action='store_true', help='Use this if the --query-re regular expression matches unique names instead of names in the database.')

    args = parser.parse_args()

    ti = TripalInstance(args.tripal, args.username, args.password)

    params = ti.analysis.getBasePayload(args)

    params.update({
        'type': 'chado_analysis_interpro',
        'interprofile': args.interpro,
        'parsego': int(args.parse_go),
        'interproparameters': args.interpro_parameters,
        'query_re': args.query_re,
        'query_type': args.query_type,
        'query_uniquename': args.query_uniquename,
    })

    res = ti.analysis.addAnalysis(params)

    print "New Interpro analysis created with ID: %s" % res['nid']



# http://bugs.python.org/issue15881#msg170215
from setuptools import setup, find_packages

from os import listdir

setup(
    name="tripal",
    version='1.3',
    description="Tripal API library",
    author="Anthony Bretaudeau",
    author_email="anthony.bretaudeau@inra.fr",
    url="https://github.com/abretaud/python-tripal",
    install_requires=['requests>=2.4.3', 'chado'],
    packages=find_packages(),
    license='MIT',
    platforms="Posix; MacOS X; Windows",
    scripts=['scripts/' + f for f in listdir('scripts')],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7"
    ])



import requests
import json
import collections
import logging
from datetime import datetime
logging.getLogger("requests").setLevel(logging.CRITICAL)
log = logging.getLogger()


def TripalAuth(parser):
    parser.add_argument('tripal', help='Complete Tripal URL')
    parser.add_argument('username', help='Admin Username')
    parser.add_argument('password', help='Admin Password')

def TripalAnalysis(parser):
    parser.add_argument("--analysis-name", required=True, help="Analysis name")
    parser.add_argument("--analysis-program", required=True, help="Program name")
    parser.add_argument("--analysis-program-version", required=True, help="Program version")
    parser.add_argument("--analysis-algorithm", help="Algorithm name")
    parser.add_argument("--analysis-source-name", required=True, help="Source name")
    parser.add_argument("--analysis-source-version", help="Source version")
    parser.add_argument("--analysis-description", help="Analysis description")
    parser.add_argument("--analysis-source-uri", help="Source URI")
    parser.add_argument("--analysis-date-executed", help="Date of execution of the analysis (format=YYYY-MM-DD, default=today)")

class TripalInstance(object):

    def __init__(self, url, username, password):
        self.tripal_url = url
        self.username = username
        self.password = password

        self.jobs = JobsClient(self)
        self.analysis = AnalysisClient(self)
        self.organism = OrganismClient(self)

    def __str__(self):
        return '<TripalInstance at %s>' % self.tripal_url

class Client(object):

    def __init__(self, tripalinstance, **requestArgs):
        self._tripal = tripalinstance

        self.__verify = requestArgs.get('verify', True)
        self._requestArgs = requestArgs
        self._session = None

        if 'verify' in self._requestArgs:
            del self._requestArgs['verify']

    def getCsrfToken(self):

        url = self._tripal.tripal_url + self.CLIENT_BASE

        headers = {
            'Content-Type': 'application/json'
        }

        if self._session:
            headers['Cookie'] = self._session

        # First get a CSRF token
        r = requests.post(url + 'user/token.json', headers=headers,
                          verify=self.__verify, **self._requestArgs)

        if r.status_code == 200:
            d = r.json()
            return d['token']
        else:
            # @see self.body for HTTP response body
            raise Exception("Unexpected response from tripal while getting CSRF token %s: %s" %
                            (r.status_code, r.text))

    def login(self):

        # First get a CSRF token
        csrfToken = self.getCsrfToken()

        # Then login
        url = self._tripal.tripal_url + self.CLIENT_BASE

        headers = {
            'Content-Type': 'application/json',
            'X-CSRF-Token': csrfToken
        }

        data = {
            'username': self._tripal.username,
            'password': self._tripal.password,
        }

        r = requests.post(url + 'user/login.json', data=json.dumps(data), headers=headers,
                          verify=self.__verify, **self._requestArgs)

        if r.status_code == 200:
            d = r.json()
            self._session = d['session_name'] + '=' + d['sessid']
        else:
            # @see self.body for HTTP response body
            raise Exception("Unexpected response from tripal while logging in %s: %s" %
                            (r.status_code, r.text))

    def request(self, clientMethod, data, post_params={}):
        url = self._tripal.tripal_url + self.CLIENT_BASE + clientMethod

        if not self._session:
            self.login()

        # Get a CSRF token
        csrfToken = self.getCsrfToken()

        headers = {
            'Content-Type': 'application/json',
            'Cookie': self._session,
            'X-CSRF-Token': csrfToken
        }

        r = requests.post(url, data=json.dumps(data), headers=headers,
                          verify=self.__verify, params=post_params, **self._requestArgs)

        if r.status_code == 200:
            d = r.json()
            return d

        # @see self.body for HTTP response body
        raise Exception("Unexpected response from tripal %s: %s" %
                        (r.status_code, r.text))

    def get(self, clientMethod, get_params):
        url = self._tripal.tripal_url + self.CLIENT_BASE + clientMethod

        if not self._session:
            self.login()

        headers = {
            'Content-Type': 'application/json',
            'Cookie': self._session # "SESSc9c711a015d1f2624abf5bff20b25337=FQ4HGEDbfXVEeU9YTGEQMTHyE-VsspsdaNVxhFsVN0k"
        }

        r = requests.get(url, headers=headers, verify=self.__verify,
                         params=get_params, **self._requestArgs)

        if r.status_code == 200:
            d = r.json()
            return d

        # @see self.body for HTTP response body
        raise Exception("Unexpected response from tripal %s: %s" %
                        (r.status_code, r.text))


class JobsClient(Client):
    CLIENT_BASE = '/tripal_api/'

    def getJobs(self):
        return self.get('job', {})

    def getJob(self, jobId):
        return self.get('job/%s' % jobId, {})

    def addJob(self, job_name, modulename, callback, arguments, priority=10):
        data = {
            'name': job_name,
            'modulename': modulename,
            'callback': callback,
            'arguments': arguments,
            'priority': priority,
        }

        return self.request('job', data)


class AnalysisClient(Client):
    CLIENT_BASE = '/tripal_api/'

    def getAnalyses(self):
        return self.get('node', {})

    def getAnalysis(self, jobId):
        return self.get('node/%s' % jobId, {})

    def getBasePayload(self, args):
        date = datetime.today()
        if args.analysis_date_executed:
            date = datetime.strptime(args.analysis_date_executed, '%Y-%m-%d')

        return {
                'analysisname': args.analysis_name,
                'program': args.analysis_program,
                'programversion': args.analysis_program_version,
                'algorithm': args.analysis_algorithm,
                'sourcename': args.analysis_source_name,
                'sourceversion': args.analysis_source_version,
                'sourceuri': args.analysis_source_uri,
                'description': args.analysis_description,
                'timeexecuted[day]': date.strftime('%d'),
                'timeexecuted[month]': date.strftime('%m'),
                'timeexecuted[year]': date.strftime('%Y'),
            }

    def addAnalysis(self, params):

        return self.request('node', params)

class OrganismClient(Client):
    CLIENT_BASE = '/tripal_api/'

    def getOrganisms(self):
        return self.get('node', {})

    def getOrganism(self, jobId):
        return self.get('node/%s' % jobId, {})

    def addOrganism(self, params):

        return self.request('node', params)



#!/usr/bin/env python
import os
import json
import argparse
from tripal import TripalAuth, TripalInstance

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Creates an organism into Tripal')
    TripalAuth(parser)
    parser.add_argument("--genus", required=True, help="The genus of the organism")
    parser.add_argument("--species", help="The species of the organism")
    parser.add_argument("--abbr", required=True, help="The abbreviation of the organism")
    parser.add_argument("--common", required=True, help="The common name of the organism")
    parser.add_argument("--description", help="The abbreviation of the organism")

    args = parser.parse_args()

    ti = TripalInstance(args.tripal, args.username, args.password)

    params = {
        'type': 'chado_organism',
        'genus': args.genus,
        'species': args.species,
        'abbreviation': args.abbr,
        'common_name': args.common,
        'description': args.description,
    }

    res = ti.organism.addOrganism(params)

    print "New organism created with Node ID: %s" % res['nid']
